classdef Converter
   
   properties (Constant)
      
      patterns = {...
         '-?\d+\.?\d*(?=cm)',... % centimeters
         '-?\d+\.?\d*(?=mm)',... % millimeters
         '-?\d+\.?\d*(?=in)',... % inches
         '-?\d+\.?\d*(?=pt)'... % points
         }
      
      inchScales = {...
         1/2.54,... % centimeters
         1/25.4,... % millimeters
         1,... % inches
         1/72 ... % points
         }
      
      perPattern = '-?\d+\.?\d*(?=%)'
      pxPattern = '-?\d+\.?\d*(?=px)'
      nonePattern = '-?\d+\.?\d*$'
      
      namePattern = '[rgbcmykw]'
      rgbPattern = '\[.+?\]'
      hexPattern = '#[0-9a-fA-F]+'
      
      
   end
   
   methods (Static)
      
      
      function values = standardizeLengths(original)
         
         import MSS.*
         
         if ischar(original)
            stringValues = regexp(original,'(?<=\s*)\S+(?=\s*)','match');
            values = cell(size(stringValues));
            for k = 1:length(stringValues)
               values{k} = Converter.standardizeSingleLength(stringValues{k});
            end
         elseif isnumeric(original)
            values = cell(size(original));
            for k = 1:numel(original)
               if isreal(original(k))
                  original(k) = round(original(k));
               end
               values{k} = original(k);
            end
         else
            error('Invalid dimension')
         end
      end
      
      
      function value = standardizeSingleLength(string)
         import MSS.*
         
         value = NaN;
         if isequal(string,'auto')
            
            value = string;
            
         elseif regexp(string,Converter.perPattern)
            
            error('Percentage lengths are not supported yet.')
            
            % Percentages are imaginary numbers in [0,1]
            percent = str2num(regexp( string, Converter.perPattern, 'match', 'once' ));
            value = percent*sqrt(-1)/100;
            
         elseif regexp(string,Converter.pxPattern) 
            
            
            % Pixels are rounded
            pixels = str2num(regexp( string, Converter.pxPattern, 'match', 'once' ));
            value = round(pixels);
         elseif regexp(string,Converter.nonePattern)
            
            % Pixels are rounded
            pixels = str2num(regexp( string, Converter.nonePattern, 'match', 'once' ));
            value = round(pixels);
            
         else
            
            % Other dimensions are scaled to pixels
            for k = 1:length(Converter.patterns)
               if regexp(string,Converter.patterns{k})
                  original = str2num(regexp( string, Converter.patterns{k}, 'match', 'once' ));
                  value = original * Converter.inchScales{k} * get(0,'ScreenPixelsPerInch');
                  value = round(value);
               end
            end
            
         end
         
         assert(not(any(isnan(value))),'Cannot interpret dimension string ''%s''',string);
         
      end % function
      
      
      function values = standardizeColors(original)
         
         import MSS.*
         
         if ischar(original)
            
            stringValues = regexp(original,'([rgbcmykw]|\[.+?\]|#[0-9a-fA-F]+|none|transparent)','match');
            values = cell(size(stringValues));
            for k = 1:length(stringValues)
               values{k} = Converter.standardizeSingleColor(stringValues{k});
            end
            
         elseif isnumeric(original)
            
            if isequal(size(original),[1 3]) && ...
               all(isreal(original)) && ...
               all(0 <= original) && all(original <= 1)
               
               values = {original};
            else
               error('Invalid color')
            end
            
         end
         
      end % function
      
      
      function value = standardizeSingleColor(string)
         
         import MSS.*
         
         value = '';
         
         if ismember(string,{'none','transparent'})
            
            value = string;
            
         elseif regexp(string,Converter.namePattern)
            
            value = regexp(string,Converter.namePattern,'match','once');
            
         elseif regexp(string,Converter.rgbPattern)
            
            try
               array = eval(string);
               if isequal(size(array),[1 3]) && ...
                  all(isreal(array)) && ...
                  all(0 <= array) && all(array <= 1)
               
                  value = array;
               else
                  value = '';
               end
            catch
               value = '';
            end
            
         elseif regexp(string,Converter.hexPattern)
            
            hexValue = regexp(string,Converter.hexPattern,'match','once');
            if length(hexValue) == 4
               r = hex2dec(char(hexValue(2)*[1 1]));
               g = hex2dec(char(hexValue(3)*[1 1]));
               b = hex2dec(char(hexValue(4)*[1 1]));
               value = [r g b]/255;
            elseif length(hexValue) == 7
               r = hex2dec(hexValue(2:3));
               g = hex2dec(hexValue(4:5));
               b = hex2dec(hexValue(6:7));
               value = [r g b]/255;
            else
               value = '';
            end
            
         else
            
            assert(isempty(value),'Cannot interpret color string ''%s''',string);
            
         end % if
         
      end % function
      
      
   end % methods
   
end % classdef