classdef Selector
   
   
   
   properties
      
      selectorString = '';
      groups = {};
      relations = {};
      
   end
   
   properties (Constant)
      
      groupPattern = '[\s>+]?\s*(?# At most one relation)([\.#]?\w+)++(?# One or more type/tag/clique, as many as possible)$(?# Look at the end of the string)';
      selectorPattern = '^([\.#]?\w+)+(?# Starts with a valid group)\s*([\s>+](?# Relational operator)\s*([\.#]?\w+)+(?# Valid group)\s*)*$(?# Continues relation-group pairs until end)';
      
   end
   
   
   
   methods
      
      
      function obj = Selector(selectorString)
         
         assert(ischar(selectorString),'Input must be a string describing a selector')
         
         if isequal(selectorString,'*')
            obj.groups = {{'*'}};
         else
         
            obj.selectorString = regexp(selectorString,obj.selectorPattern,'match','once');

            if not(isempty(obj.selectorString))
               obj = obj.parseSelectorString();
            end
            
         end
         
      end % function
      
      
      function obj = parseSelectorString(obj)
         
         % Parse selector text from right to left.
         % At each point, operate on selectText(1:p).
         p = length(obj.selectorString);
         
         while (p > 0)
            
            % Trim off trailing whitespace
            p = regexp(obj.selectorString(1:p),'\S\s*$','start');
            
            % Parse right-most group
            [group,start] = regexp(obj.selectorString(1:p),obj.groupPattern,'match','start','once');
            obj = obj.parseGroupString(group);
            
            % Trim off parsed group
            p = start - 1;
            
         end
         
      end % function
      
      
      function obj = parseGroupString(obj,groupString)
         
         % Break group into type/tag/clique pieces
         obj.groups{end+1} = regexp(groupString,'[\.#]?\w+','match');
         
         % Record relation to preceding group (to the left)
         if ismember(groupString(1),{' ','>','+'})
            obj.relations{end+1} = groupString(1);
         end
         
      end % function
      
      
      function elements = selectElements(obj,tree)
         
         n = length(obj.groups);
         elements = obj.selectElementGroup(tree,obj.groups{1});
         for k = 1:n-1
            relatives = obj.selectElementGroup(tree,obj.groups{k+1});
            elements = obj.applyRelation(elements,relatives,obj.relations{k});
         end
         
      end % function
      
      
      function elements = selectElementGroup(obj,tree,group)
         
         sortedTree = tree.topologicalSort();
         elements = {};
         for k = 1:length(sortedTree)
            if obj.isGroupMember(sortedTree{k},group)
               elements{end+1} = sortedTree{k};
            end
         end
         
      end % function
      
      
      function reply = isGroupMember(obj,element,group)
         
         reply = true;
         for k = 1:length(group)
            part = group{k};
            if isequal(part(1),'.')
               reply = reply && element.isInClique(part(2:end));
            elseif isequal(part(1),'#')
               reply = reply && isequal(element.tag,part(2:end));
            else
               reply = reply && isequal(element.type,part);
            end
         end
         
      end % function
      
      
      function elements = applyRelation(obj,candidates,relatives,relation)
         
         elements = {};
         
         for j = 1:length(candidates)
            
            can = candidates{j};
            keep = false;
            
            for k = 1:length(relatives)
               
               rel = relatives{k};
               
               switch relation
                  case ' '
                     keep = keep || rel.hasDescendent(can);
                  case '>'
                     keep = keep || rel.hasChild(can);
                  case '+'
                     keep = keep || isequal(rel.getNextSibling(),can);
               end
            
               % Stop searching relatives if relation is true.
               if keep
                  elements{end+1} = can;
                  break;
               end
               
            end % for k
            
         end % for j
         
      end % function
      
      
      function display(objs)
         
         for j = 1:length(objs)
            
            
            fprintf('\n%s',objs(j).toString());

            
         end % for j
         
         fprintf('\n\n')
         
      end % function
      
      
      function string = toString(obj)
         string = '';
         for k = length(obj.groups):-1:1
            
            string = [string,obj.groups{k}{:}];
            
            if (k>1)
               if isequal(obj.relations{k-1},' ')
                  string = [string,' '];
               else
                  string = [string,sprintf(' %s ',obj.relations{k-1})];
               end
            end
            
         end % for k
         
      end % function
      
      
   end % methods
   
   
   
end % classdef